@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -56,9 +56,9 @@ from nuitka.Progress import (
     setupProgressBar,
 )
 from nuitka.PythonFlavors import (
-    isAnacondaPython,
     isApplePython,
     isDebianPackagePython,
+    isFedoraPackagePython,
     isMSYS2MingwPython,
     isNuitkaPython,
     isPyenvPython,
@@ -76,7 +76,6 @@ from nuitka.utils import InstanceCounters, MemoryUsage
 from nuitka.utils.Execution import (
     callProcess,
     withEnvironmentVarOverridden,
-    withEnvironmentVarsOverridden,
     wrapCommandForDebuggerForExec,
 )
 from nuitka.utils.FileOperations import (
@@ -84,8 +83,7 @@ from nuitka.utils.FileOperations import (
     deleteFile,
     getDirectoryRealPath,
     getExternalUsePath,
-    makePath,
-    putTextFileContents,
+    openTextFile,
     removeDirectory,
     resetDirectory,
 )
@@ -96,16 +94,28 @@ from nuitka.utils.StaticLibraries import getSystemStaticLibPythonPath
 from nuitka.utils.Utils import getArchitecture, isMacOS, isWin32Windows
 from nuitka.Version import getCommercialVersion, getNuitkaVersion
 
-from . import ModuleRegistry, Options, OutputDirectories, TreeXML
-from .build import SconsInterface
-from .codegen import CodeGeneration, LoaderCodes, Reports
+from . import ModuleRegistry, Options, OutputDirectories
+from .build.SconsInterface import (
+    asBoolStr,
+    cleanSconsDirectory,
+    getSconsDataPath,
+    runScons,
+    setCommonSconsOptions,
+)
+from .code_generation import CodeGeneration, LoaderCodes, Reports
 from .finalizations import Finalization
 from .freezer.Onefile import packDistFolderToOnefile
-from .freezer.Standalone import copyDllsUsed
+from .freezer.Standalone import (
+    checkFreezingModuleSet,
+    copyDllsUsed,
+    detectUsedDLLs,
+)
 from .optimizations.Optimization import optimizeModules
 from .pgo.PGO import readPGOInputFile
 from .Reports import writeCompilationReport
-from .tree import Building
+from .tree.Building import buildMainModuleTree
+from .tree.SourceHandling import writeSourceCode
+from .TreeXML import dumpTreeXMLToFile
 
 
 def _createNodeTree(filename):
@@ -120,7 +130,7 @@ def _createNodeTree(filename):
     # Many cases to deal with, pylint: disable=too-many-branches
 
     # First, build the raw node tree from the source code.
-    main_module = Building.buildMainModuleTree(
+    main_module = buildMainModuleTree(
         filename=filename,
         is_main=not Options.shallMakeModule(),
     )
@@ -131,7 +141,7 @@ def _createNodeTree(filename):
     source_dir = OutputDirectories.getSourceDirectoryPath()
 
     if not Options.shallOnlyExecCCompilerCall():
-        SconsInterface.cleanSconsDirectory(source_dir)
+        cleanSconsDirectory(source_dir)
 
     # Prepare the ".dist" directory, throwing away what was there before.
     if Options.isStandaloneMode():
@@ -169,8 +179,8 @@ def _createNodeTree(filename):
 
         if kind != "absolute":
             inclusion_logger.sysexit(
-                "Error, failed to locate package %r you asked to include."
-                % package_name.asString()
+                "Error, failed to locate package '%s' you asked to include."
+                % package_name
             )
 
         Recursion.checkPluginPath(
@@ -201,6 +211,10 @@ def _createNodeTree(filename):
     # Then optimize the tree and potentially recursed modules.
     optimizeModules(main_module.getOutputFilename())
 
+    # Freezer may have concerns for some modules.
+    if Options.isStandaloneMode():
+        checkFreezingModuleSet()
+
     # Allow plugins to comment on final module set.
     Plugins.onModuleCompleteSet()
 
@@ -215,9 +229,14 @@ def _createNodeTree(filename):
         return main_module
 
 
-def dumpTreeXML(tree):
-    xml_root = tree.asXml()
-    TreeXML.dump(xml_root)
+def dumpTreeXML():
+    filename = Options.getXMLDumpOutputFilename()
+
+    if filename is not None:
+        with openTextFile(filename, "w") as output_file:
+            # XML output only.
+            for module in ModuleRegistry.getDoneModules():
+                dumpTreeXMLToFile(tree=module.asXml(), output_file=output_file)
 
 
 def pickSourceFilenames(source_dir, modules):
@@ -292,9 +311,6 @@ def pickSourceFilenames(source_dir, modules):
     return module_filenames
 
 
-standalone_entry_points = []
-
-
 def makeSourceDirectory():
     """Get the full list of modules imported, create code for all of them."""
     # We deal with a lot of details here, but rather one by one, and split makes
@@ -334,10 +350,9 @@ def makeSourceDirectory():
         if "*" in any_case_module or "{" in any_case_module:
             continue
 
-        for module in ModuleRegistry.getDoneModules():
-            if module.getFullName() == any_case_module:
-                break
-        else:
+        if not ModuleRegistry.hasDoneModule(
+            any_case_module
+        ) and not ModuleRegistry.hasRootModule(any_case_module):
             general.warning(
                 "Did not follow import to unused '%s', consider include options."
                 % any_case_module
@@ -398,7 +413,7 @@ def makeSourceDirectory():
 
         source_code = CodeGeneration.generateModuleCode(
             module=module,
-            data_filename=os.path.basename(c_filename + "onst"),  # Really .const
+            data_filename=os.path.basename(c_filename[:-2] + ".const"),
         )
 
         writeSourceCode(filename=c_filename, source_code=source_code)
@@ -458,11 +473,11 @@ def _wasMsvcMode():
 def _deleteMsvcPGOFiles(pgo_mode):
     assert _wasMsvcMode()
 
-    msvc_pgc_filename = OutputDirectories.getResultBasepath(onefile=False) + "!1.pgc"
+    msvc_pgc_filename = OutputDirectories.getResultBasePath(onefile=False) + "!1.pgc"
     deleteFile(msvc_pgc_filename, must_exist=False)
 
     if pgo_mode == "use":
-        msvc_pgd_filename = OutputDirectories.getResultBasepath(onefile=False) + ".pgd"
+        msvc_pgd_filename = OutputDirectories.getResultBasePath(onefile=False) + ".pgd"
         deleteFile(msvc_pgd_filename, must_exist=False)
 
     return msvc_pgc_filename
@@ -492,6 +507,7 @@ def _runCPgoBinary():
     else:
         _exit_code = _runPgoBinary()
 
+        # gcc file suffix, spell-checker: ignore gcda
         gcc_constants_pgo_filename = os.path.join(
             OutputDirectories.getSourceDirectoryPath(), "__constants.gcda"
         )
@@ -529,22 +545,20 @@ def runSconsBackend(quiet):
     # have checks for them, leading to many branches and statements,
     # pylint: disable=too-many-branches,too-many-statements
 
-    asBoolStr = SconsInterface.asBoolStr
-
     options = {
-        "result_name": OutputDirectories.getResultBasepath(onefile=False),
+        "result_name": OutputDirectories.getResultBasePath(onefile=False),
         "source_dir": OutputDirectories.getSourceDirectoryPath(),
         "nuitka_python": asBoolStr(isNuitkaPython()),
         "debug_mode": asBoolStr(Options.is_debug),
         "python_debug": asBoolStr(Options.shallUsePythonDebug()),
         "module_mode": asBoolStr(Options.shallMakeModule()),
-        "full_compat": asBoolStr(Options.is_fullcompat),
+        "full_compat": asBoolStr(Options.is_full_compat),
         "experimental": ",".join(Options.getExperimentalIndications()),
         "trace_mode": asBoolStr(Options.shallTraceExecution()),
         "python_version": python_version_str,
         "target_arch": getArchitecture(),
         "python_prefix": getDirectoryRealPath(getSystemPrefixPath()),
-        "nuitka_src": SconsInterface.getSconsDataPath(),
+        "nuitka_src": getSconsDataPath(),
         "module_count": "%d"
         % (
             1
@@ -572,12 +586,12 @@ def runSconsBackend(quiet):
     if isDebianPackagePython():
         options["debian_python"] = asBoolStr(True)
 
+    if isFedoraPackagePython():
+        options["fedora_python"] = asBoolStr(True)
+
     if isMSYS2MingwPython():
         options["msys2_mingw_python"] = asBoolStr(True)
 
-    if isAnacondaPython():
-        options["anaconda_python"] = asBoolStr(True)
-
     if isApplePython():
         options["apple_python"] = asBoolStr(True)
 
@@ -652,15 +666,21 @@ def runSconsBackend(quiet):
     if Options.shallMakeModule():
         options["module_suffix"] = getSharedLibrarySuffix(preferred=True)
 
-    env_values = SconsInterface.setCommonOptions(options)
+    env_values = setCommonSconsOptions(options)
+
+    # Allow plugins to build definitions.
+    env_values.update(Plugins.getBuildDefinitions())
 
     if Options.shallCreatePgoInput():
         options["pgo_mode"] = "python"
 
-        with withEnvironmentVarsOverridden(env_values):
-            result = SconsInterface.runScons(
-                options=options, quiet=quiet, scons_filename="Backend.scons"
-            )
+        result = runScons(
+            options=options,
+            quiet=quiet,
+            env_values=env_values,
+            scons_filename="Backend.scons",
+        )
+
         if not result:
             return result, options
 
@@ -678,10 +698,12 @@ def runSconsBackend(quiet):
         if Options.isPgoMode():
             options["pgo_mode"] = "generate"
 
-            with withEnvironmentVarsOverridden(env_values):
-                result = SconsInterface.runScons(
-                    options=options, quiet=quiet, scons_filename="Backend.scons"
-                )
+            result = runScons(
+                options=options,
+                quiet=quiet,
+                env_values=env_values,
+                scons_filename="Backend.scons",
+            )
 
             if not result:
                 return result, options
@@ -691,39 +713,23 @@ def runSconsBackend(quiet):
             _runCPgoBinary()
             options["pgo_mode"] = "use"
 
-    with withEnvironmentVarsOverridden(env_values):
-        result = (
-            SconsInterface.runScons(
-                options=options, quiet=quiet, scons_filename="Backend.scons"
-            ),
-            options,
-        )
+    result = (
+        runScons(
+            options=options,
+            quiet=quiet,
+            env_values=env_values,
+            scons_filename="Backend.scons",
+        ),
+        options,
+    )
 
+    # Delete PGO files if asked to do that.
     if options.get("pgo_mode") == "use" and _wasMsvcMode():
         _deleteMsvcPGOFiles(pgo_mode="use")
 
     return result
 
 
-def writeSourceCode(filename, source_code):
-    # Prevent accidental overwriting. When this happens the collision detection
-    # or something else has failed.
-    assert not os.path.isfile(filename), filename
-
-    putTextFileContents(filename=filename, contents=source_code, encoding="latin1")
-
-
-def writeBinaryData(filename, binary_data):
-    # Prevent accidental overwriting. When this happens the collision detection
-    # or something else has failed.
-    assert not os.path.isfile(filename), filename
-
-    assert type(binary_data) is bytes
-
-    with open(filename, "wb") as output_file:
-        output_file.write(binary_data)
-
-
 def callExecPython(args, clean_path, add_path):
     old_python_path = os.environ.get("PYTHONPATH")
 
@@ -753,6 +759,7 @@ def executeMain(binary_filename, clean_path):
 
 
 def executeModule(tree, clean_path):
+    """Execute the extension module just created."""
 
     if python_version < 0x340:
         python_command_template = """\
@@ -810,7 +817,7 @@ def compileTree():
 
         # This should take all bytecode values, even ones needed for frozen or
         # not produce anything.
-        loader_code = LoaderCodes.getMetapathLoaderBodyCode(bytecode_accessor)
+        loader_code = LoaderCodes.getMetaPathLoaderBodyCode(bytecode_accessor)
 
         writeSourceCode(
             filename=os.path.join(source_dir, "__loader.c"), source_code=loader_code
@@ -842,15 +849,7 @@ def compileTree():
 
     runDataComposer(source_dir)
 
-    for filename, source_code in Plugins.getExtraCodeFiles().items():
-        target_dir = os.path.join(source_dir, "plugins")
-
-        if not os.path.isdir(target_dir):
-            makePath(target_dir)
-
-        writeSourceCode(
-            filename=os.path.join(target_dir, filename), source_code=source_code
-        )
+    Plugins.writeExtraCodeFiles(onefile=False)
 
     general.info("Running C compilation via Scons.")
 
@@ -866,7 +865,7 @@ def handleSyntaxError(e):
     # versions he wants, tell him about the potential version problem.
     error_message = SyntaxErrors.formatOutput(e)
 
-    if not Options.is_fullcompat:
+    if not Options.is_full_compat:
         if python_version < 0x300:
             suggested_python_version_str = getSupportedPythonVersions()[-1]
         else:
@@ -906,17 +905,17 @@ def main():
     if pgo_filename is not None:
         readPGOInputFile(pgo_filename)
 
-    if not Options.shallDumpBuiltTreeXML():
-        general.info(
-            "Starting Python compilation with Nuitka %r on Python %r commercial %r."
-            % (getNuitkaVersion(), python_version_str, getCommercialVersion())
+    general.info(
+        "Starting Python compilation with Nuitka %r on Python %r commercial grade %r."
+        % (
+            getNuitkaVersion(),
+            python_version_str,
+            getCommercialVersion() or "not installed",
         )
+    )
 
     filename = Options.getPositionalArgs()[0]
 
-    if not os.path.exists(filename):
-        general.sysexit("Error, file '%s' is not found." % filename)
-
     # Inform the importing layer about the main script directory, so it can use
     # it when attempting to follow imports.
     Importing.setMainScriptDirectory(
@@ -933,114 +932,134 @@ def main():
 
     addIncludedDataFilesFromPackageOptions()
 
-    if Options.shallDumpBuiltTreeXML():
-        # XML output only.
+    dumpTreeXML()
+
+    # Make the actual compilation.
+    result, options = compileTree()
+
+    # Exit if compilation failed.
+    if not result:
+        sys.exit(1)
+
+    # Relaunch in case of Python PGO input to be produced.
+    if Options.shallCreatePgoInput():
+        # Will not return.
+        pgo_filename = OutputDirectories.getPgoRunInputFilename()
+        general.info(
+            "Restarting compilation using collected information from '%s'."
+            % pgo_filename
+        )
+        reExecuteNuitka(pgo_filename=pgo_filename)
+
+    if Options.shallNotDoExecCCompilerCall():
+        if Options.isShowMemory():
+            MemoryUsage.showMemoryTrace()
+
+        sys.exit(0)
+
+    executePostProcessing()
+
+    copyDataFiles()
+
+    if Options.isStandaloneMode():
+        binary_filename = options["result_exe"]
+
+        setMainEntryPoint(binary_filename)
+
         for module in ModuleRegistry.getDoneModules():
-            dumpTreeXML(module)
-    else:
-        # Make the actual compilation.
-        result, options = compileTree()
+            addIncludedEntryPoints(Plugins.considerExtraDlls(module))
 
-        # Exit if compilation failed.
-        if not result:
-            sys.exit(1)
-
-        # Relaunch in case of Python PGO input to be produced.
-        if Options.shallCreatePgoInput():
-            # Will not return.
-            pgo_filename = OutputDirectories.getPgoRunInputFilename()
-            general.info(
-                "Restarting compilation using collected information from '%s'."
-                % pgo_filename
-            )
-            reExecuteNuitka(pgo_filename=pgo_filename)
-
-        if Options.shallNotDoExecCCompilerCall():
-            if Options.isShowMemory():
-                MemoryUsage.showMemoryTrace()
-
-            sys.exit(0)
-
-        executePostProcessing()
-
-        copyDataFiles()
-
-        if Options.isStandaloneMode():
-            binary_filename = options["result_exe"]
-
-            setMainEntryPoint(binary_filename)
-
-            dist_dir = OutputDirectories.getStandaloneDirectoryPath()
-
-            for module in ModuleRegistry.getDoneModules():
-                addIncludedEntryPoints(Plugins.considerExtraDlls(dist_dir, module))
-
-            copyDllsUsed(
-                source_dir=OutputDirectories.getSourceDirectoryPath(),
-                dist_dir=dist_dir,
-                standalone_entry_points=getStandaloneEntryPoints(),
-            )
-
-            Plugins.onStandaloneDistributionFinished(dist_dir)
-
-            if Options.isOnefileMode():
-                packDistFolderToOnefile(dist_dir, binary_filename)
-
-                if Options.isRemoveBuildDir():
-                    general.info("Removing dist folder %r." % dist_dir)
-
-                    removeDirectory(path=dist_dir, ignore_errors=False)
-                else:
-                    general.info(
-                        "Keeping dist folder %r for inspection, no need to use it."
-                        % dist_dir
-                    )
-
-        # Remove the source directory (now build directory too) if asked to.
-        source_dir = OutputDirectories.getSourceDirectoryPath()
-
-        if Options.isRemoveBuildDir():
-            general.info("Removing build directory %r." % source_dir)
-
-            removeDirectory(path=source_dir, ignore_errors=False)
-            assert not os.path.exists(source_dir)
-        else:
-            general.info("Keeping build directory %r." % source_dir)
-
-        final_filename = OutputDirectories.getResultFullpath(
-            onefile=Options.isOnefileMode()
+        detectUsedDLLs(
+            standalone_entry_points=getStandaloneEntryPoints(),
+            source_dir=OutputDirectories.getSourceDirectoryPath(),
         )
 
-        if Options.isStandaloneMode() and isMacOS():
-            general.info(
-                "Created binary that runs on macOS %s (%s) or higher."
-                % (options["macos_min_version"], options["macos_target_arch"])
-            )
+        dist_dir = OutputDirectories.getStandaloneDirectoryPath()
 
-        Plugins.onFinalResult(final_filename)
+        copyDllsUsed(
+            dist_dir=dist_dir,
+            standalone_entry_points=getStandaloneEntryPoints(),
+        )
 
-        general.info("Successfully created %r." % final_filename)
+        Plugins.onStandaloneDistributionFinished(dist_dir)
 
-        report_filename = Options.getCompilationReportFilename()
+        if Options.isOnefileMode():
+            packDistFolderToOnefile(dist_dir)
 
-        if report_filename:
-            writeCompilationReport(report_filename)
+            if Options.isRemoveBuildDir():
+                general.info("Removing dist folder %r." % dist_dir)
 
-        # Execute the module immediately if option was given.
-        if Options.shallExecuteImmediately():
-            run_filename = OutputDirectories.getResultRunFilename(
-                onefile=Options.isOnefileMode()
-            )
-
-            general.info("Launching %r." % run_filename)
-
-            if Options.shallMakeModule():
-                executeModule(
-                    tree=main_module,
-                    clean_path=Options.shallClearPythonPathEnvironment(),
-                )
+                removeDirectory(path=dist_dir, ignore_errors=False)
             else:
-                executeMain(
-                    binary_filename=run_filename,
-                    clean_path=Options.shallClearPythonPathEnvironment(),
+                general.info(
+                    "Keeping dist folder '%s' for inspection, no need to use it."
+                    % dist_dir
                 )
+
+    # Remove the source directory (now build directory too) if asked to.
+    source_dir = OutputDirectories.getSourceDirectoryPath()
+
+    if Options.isRemoveBuildDir():
+        general.info("Removing build directory '%s'." % source_dir)
+
+        removeDirectory(path=source_dir, ignore_errors=False)
+        assert not os.path.exists(source_dir)
+    else:
+        general.info("Keeping build directory '%s'." % source_dir)
+
+    final_filename = OutputDirectories.getResultFullpath(
+        onefile=Options.isOnefileMode()
+    )
+
+    if Options.isStandaloneMode() and isMacOS():
+        general.info(
+            "Created binary that runs on macOS %s (%s) or higher."
+            % (options["macos_min_version"], options["macos_target_arch"])
+        )
+
+    Plugins.onFinalResult(final_filename)
+
+    if Options.shallMakeModule():
+        base_path = OutputDirectories.getResultBasePath(onefile=False)
+
+        if os.path.isdir(base_path) and os.path.isfile(
+            os.path.join(base_path, "__init__.py")
+        ):
+            general.warning(
+                """\
+The compilation result is hidden by package directory '%s'. Importing will \
+not use compiled code while it exists."""
+                % base_path
+            )
+
+    general.info("Successfully created '%s'." % final_filename)
+
+    report_filename = Options.getCompilationReportFilename()
+
+    if report_filename:
+        writeCompilationReport(report_filename)
+
+    run_filename = OutputDirectories.getResultRunFilename(
+        onefile=Options.isOnefileMode()
+    )
+
+    # Execute the module immediately if option was given.
+    if Options.shallExecuteImmediately():
+        general.info("Launching '%s'" % run_filename)
+
+        if Options.shallMakeModule():
+            executeModule(
+                tree=main_module,
+                clean_path=Options.shallClearPythonPathEnvironment(),
+            )
+        else:
+            executeMain(
+                binary_filename=run_filename,
+                clean_path=Options.shallClearPythonPathEnvironment(),
+            )
+    else:
+        if run_filename != final_filename:
+            general.info(
+                "Execute it by launching '%s', the batch file needs to set environment."
+                % run_filename
+            )
