@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -334,20 +334,21 @@ def executePostProcessing():
         python_abi_version = python_version_str + getPythonABI()
         python_dll_filename = "libpython" + python_abi_version + ".dylib"
         python_lib_path = os.path.join(sys.prefix, "lib")
+        python_dll_path = os.path.join(python_lib_path, python_dll_filename)
 
-        # Note: For CPython and potentially others, the rpath for the Python
-        # library needs to be set.
-
+        # Note: For CPython, and potentially others, the rpath for the Python
+        # library needs to be set, so it will be detected as a dependency
+        # without tricks.
         callInstallNameTool(
             filename=result_filename,
             mapping=(
                 (
                     python_dll_filename,
-                    os.path.join(python_lib_path, python_dll_filename),
+                    python_dll_path,
                 ),
                 (
                     "@rpath/Python3.framework/Versions/%s/Python3" % python_version_str,
-                    os.path.join(python_lib_path, python_dll_filename),
+                    python_dll_path,
                 ),
             ),
             id_path=None,
@@ -381,13 +382,14 @@ def executePostProcessing():
 @echo off
 rem This script was created by Nuitka to execute '%(exe_filename)s' with Python DLL being found.
 set PATH=%(dll_directory)s;%%PATH%%
-set PYTHONHOME=%(dll_directory)s
+set PYTHONHOME=%(python_home)s
 %(debugger_call)s"%%~dp0.\\%(exe_filename)s" %%*
 """ % {
             "debugger_call": (" ".join(wrapCommandForDebuggerForExec()) + " ")
             if Options.shallRunInDebugger()
             else "",
             "dll_directory": dll_directory,
+            "python_home": sys.prefix,
             "exe_filename": os.path.basename(result_filename),
         }
 
@@ -395,7 +397,7 @@ set PYTHONHOME=%(dll_directory)s
 
     # Create a ".pyi" file for created modules
     if Options.shallMakeModule() and Options.shallCreatePyiFile():
-        pyi_filename = OutputDirectories.getResultBasepath() + ".pyi"
+        pyi_filename = OutputDirectories.getResultBasePath() + ".pyi"
 
         putTextFileContents(
             filename=pyi_filename,
