@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,13 +22,6 @@ source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementAssignmentVariableName,
-    StatementDelVariable,
-    StatementDelVariableName,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.AttributeNodes import (
     StatementAssignmentAttribute,
     StatementDelAttribute,
@@ -74,11 +67,16 @@ from nuitka.nodes.SubscriptNodes import (
     StatementAssignmentSubscript,
     StatementDelSubscript,
 )
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTempVariableRef,
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableDelNodes import makeStatementDelVariable
+from nuitka.nodes.VariableNameNodes import (
     ExpressionVariableLocalNameRef,
     ExpressionVariableNameRef,
+    StatementAssignmentVariableName,
+    StatementDelVariableName,
 )
+from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.Options import hasPythonFlagNoAnnotations, isExperimental
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import general
@@ -239,7 +237,7 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source, source_
             if starred_list_var is not None:
                 statements.insert(
                     starred_index + 1,
-                    StatementAssignmentVariable(
+                    makeStatementAssignmentVariable(
                         variable=element_var,
                         source=ExpressionListOperationPop(
                             list_arg=ExpressionTempVariableRef(
@@ -252,7 +250,7 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source, source_
                 )
             elif element[0] != "Starred":
                 statements.append(
-                    StatementAssignmentVariable(
+                    makeStatementAssignmentVariable(
                         variable=element_var,
                         source=ExpressionSpecialUnpack(
                             value=ExpressionTempVariableRef(
@@ -271,7 +269,7 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source, source_
                 starred_list_var = element_var
 
                 statements.append(
-                    StatementAssignmentVariable(
+                    makeStatementAssignmentVariable(
                         variable=element_var,
                         source=ExpressionBuiltinList(
                             value=ExpressionTempVariableRef(
@@ -342,7 +340,7 @@ not enough values to unpack (expected at least %d, got %%d)"""
             iter_creation_class = ExpressionBuiltinIter1
 
         statements = [
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=source_iter_var,
                 source=iter_creation_class(value=source, source_ref=source_ref),
                 source_ref=source_ref,
@@ -351,7 +349,7 @@ not enough values to unpack (expected at least %d, got %%d)"""
                 provider=provider,
                 tried=statements,
                 final=(
-                    StatementReleaseVariable(
+                    makeStatementReleaseVariable(
                         variable=source_iter_var, source_ref=source_ref
                     ),
                 ),
@@ -382,7 +380,7 @@ not enough values to unpack (expected at least %d, got %%d)"""
             # Need to release temporary variables right after successful
             # usage.
             statements.append(
-                StatementDelVariable(
+                makeStatementDelVariable(
                     variable=element_var, tolerant=True, source_ref=source_ref
                 )
             )
@@ -391,7 +389,9 @@ not enough values to unpack (expected at least %d, got %%d)"""
 
         for element_var in element_vars:
             final_statements.append(
-                StatementReleaseVariable(variable=element_var, source_ref=source_ref)
+                makeStatementReleaseVariable(
+                    variable=element_var, source_ref=source_ref
+                )
             )
 
         return makeTryFinallyStatement(
@@ -563,7 +563,7 @@ def buildAssignNode(provider, node, source_ref):
         )
 
         statements = [
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_source, source=source, source_ref=source_ref
             )
         ]
@@ -583,7 +583,9 @@ def buildAssignNode(provider, node, source_ref):
         return makeTryFinallyStatement(
             provider=provider,
             tried=statements,
-            final=StatementReleaseVariable(variable=tmp_source, source_ref=source_ref),
+            final=makeStatementReleaseVariable(
+                variable=tmp_source, source_ref=source_ref
+            ),
             source_ref=source_ref,
         )
 
@@ -796,7 +798,7 @@ def _buildInplaceAssignAttributeNode(
     tmp_variable = provider.allocateTempVariable(temp_scope=temp_scope, name="value")
 
     # First assign the target value to a temporary variable.
-    preserve_to_tmp = StatementAssignmentVariable(
+    preserve_to_tmp = makeStatementAssignmentVariable(
         variable=tmp_variable,
         source=makeExpressionAttributeLookup(
             expression=lookup_source.makeClone(),
@@ -807,7 +809,7 @@ def _buildInplaceAssignAttributeNode(
     )
 
     # Second assign the in-place result to a temporary variable
-    inplace_to_tmp = StatementAssignmentVariable(
+    inplace_to_tmp = makeStatementAssignmentVariable(
         variable=tmp_variable,
         source=makeExpressionOperationBinaryInplace(
             operator=operator,
@@ -834,7 +836,7 @@ def _buildInplaceAssignAttributeNode(
         makeTryFinallyStatement(
             provider=provider,
             tried=(inplace_to_tmp, copy_back_from_tmp),
-            final=StatementReleaseVariable(
+            final=makeStatementReleaseVariable(
                 variable=tmp_variable, source_ref=source_ref
             ),
             source_ref=source_ref,
@@ -854,15 +856,15 @@ def _buildInplaceAssignSubscriptNode(
     source_ref,
 ):
     # First assign the subscribed value to a temporary variable.
-    preserve_to_tmp1 = StatementAssignmentVariable(
+    preserve_to_tmp1 = makeStatementAssignmentVariable(
         variable=tmp_variable1, source=subscribed, source_ref=source_ref
     )
     # Second assign the subscript value to a temporary variable
     statements = (
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_variable2, source=subscript, source_ref=source_ref
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_variable3,
             source=ExpressionSubscriptLookup(
                 expression=ExpressionTempVariableRef(
@@ -875,7 +877,7 @@ def _buildInplaceAssignSubscriptNode(
             ),
             source_ref=source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_variable3,
             source=makeExpressionOperationBinaryInplace(
                 operator=operator,
@@ -907,9 +909,15 @@ def _buildInplaceAssignSubscriptNode(
             provider=provider,
             tried=statements,
             final=(
-                StatementReleaseVariable(variable=tmp_variable1, source_ref=source_ref),
-                StatementReleaseVariable(variable=tmp_variable2, source_ref=source_ref),
-                StatementReleaseVariable(variable=tmp_variable3, source_ref=source_ref),
+                makeStatementReleaseVariable(
+                    variable=tmp_variable1, source_ref=source_ref
+                ),
+                makeStatementReleaseVariable(
+                    variable=tmp_variable2, source_ref=source_ref
+                ),
+                makeStatementReleaseVariable(
+                    variable=tmp_variable3, source_ref=source_ref
+                ),
             ),
             source_ref=source_ref,
         ),
@@ -935,23 +943,23 @@ def _buildInplaceAssignSliceNode(
     # pylint: disable=too-many-locals
 
     # First assign the target value, lower and upper to temporary variables.
-    copy_to_tmp = StatementAssignmentVariable(
+    copy_to_tmp = makeStatementAssignmentVariable(
         variable=tmp_variable1, source=lookup_source, source_ref=source_ref
     )
 
     final_statements = [
-        StatementReleaseVariable(variable=tmp_variable1, source_ref=source_ref)
+        makeStatementReleaseVariable(variable=tmp_variable1, source_ref=source_ref)
     ]
     statements = []
 
     if lower is not None:
         statements.append(
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_variable2, source=lower, source_ref=source_ref
             )
         )
         final_statements.append(
-            StatementReleaseVariable(variable=tmp_variable2, source_ref=source_ref)
+            makeStatementReleaseVariable(variable=tmp_variable2, source_ref=source_ref)
         )
 
         lower_ref1 = ExpressionTempVariableRef(
@@ -967,12 +975,12 @@ def _buildInplaceAssignSliceNode(
 
     if upper is not None:
         statements.append(
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_variable3, source=upper, source_ref=source_ref
             )
         )
         final_statements.append(
-            StatementReleaseVariable(variable=tmp_variable3, source_ref=source_ref)
+            makeStatementReleaseVariable(variable=tmp_variable3, source_ref=source_ref)
         )
 
         upper_ref1 = ExpressionTempVariableRef(
@@ -991,7 +999,7 @@ def _buildInplaceAssignSliceNode(
     # Second assign the in-place result over the original value.
     if use_sliceobj:
         statements += (
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_variable4,
                 source=ExpressionSubscriptLookup(
                     expression=ExpressionTempVariableRef(
@@ -1007,7 +1015,7 @@ def _buildInplaceAssignSliceNode(
                 ),
                 source_ref=source_ref,
             ),
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_variable4,
                 source=makeExpressionOperationBinaryInplace(
                     operator=operator,
@@ -1034,7 +1042,7 @@ def _buildInplaceAssignSliceNode(
         )
     else:
         statements += (
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_variable4,
                 source=ExpressionSliceLookup(
                     expression=ExpressionTempVariableRef(
@@ -1046,7 +1054,7 @@ def _buildInplaceAssignSliceNode(
                 ),
                 source_ref=source_ref,
             ),
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_variable4,
                 source=makeExpressionOperationBinaryInplace(
                     operator=operator,
@@ -1072,7 +1080,7 @@ def _buildInplaceAssignSliceNode(
         )
 
     final_statements.append(
-        StatementReleaseVariable(variable=tmp_variable4, source_ref=source_ref)
+        makeStatementReleaseVariable(variable=tmp_variable4, source_ref=source_ref)
     )
 
     return (
@@ -1219,7 +1227,7 @@ def buildNamedExprNode(provider, node, source_ref):
         )
 
     statements = (
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_value, source=value, source_ref=source_ref
         ),
         StatementAssignmentVariableName(
@@ -1242,7 +1250,7 @@ def buildNamedExprNode(provider, node, source_ref):
             statement=makeTryFinallyStatement(
                 provider=provider,
                 tried=statements,
-                final=StatementReleaseVariable(
+                final=makeStatementReleaseVariable(
                     variable=tmp_value, source_ref=source_ref
                 ),
                 source_ref=source_ref,
